#!/usr/local/bin/perl -w

use Getopt::Long;                        # for command-line switches
use Mail::POP3Client;                    # for POP functions
use Tk;                                  # for primitives.
use Tk::Dialog;                          # for dialog box


# $host is assigned to `hostname` (if -host $host is undefined)
# -help switch shows program usage message; program exits

GetOptions('host:s', \$host, 'help:s', \&usage);
chop($host = `hostname`) unless $host;

$main = new MainWindow;                   # the main container


# Top frame of ptk_mailer

$top = $main->Frame;

$sv = $top->Button(-text => 'Save', -relief => 'raised', -borderwidth => 2, 
		   -command => sub{&save_message_to_file})->pack();
$sv->pack(-side=>'left');

$dm = $top->Button(-text => 'Delete', -command => sub{&delete_message},
		   -relief => 'raised', -borderwidth => 2)->pack();
$dm->pack(-side=>'left');

$ex = $top->Button(-text => 'Exit', -command => sub{exit},
		   -relief => 'raised', -borderwidth => 2)->pack();
$ex->pack(-side=>'left');

$li = $top->Button(-text => 'Login', -relief => 'raised', 
		   -command => sub{&popup_auth_window()}, 
		   -borderwidth => 2)->pack();
$li->pack(-side=>'left');
$top->pack(-fill => 'both', -expand => 'yes');


# Middle frame of ptk_mailer

$middle = $main->Frame;
$msg_area = $middle->Listbox(-relief => 'sunken', -borderwidth => 3,
			     -width => 80, -height => 10, -setgrid => 'yes');
$msg_area->bind('<Double-1>', \&show_message_body);
$lst_scrl = $middle->Scrollbar(-command => ['yview', $msg_area]);
$msg_area->configure(-yscrollcommand => ['set', $lst_scrl]);
$msg_area->pack(-side => 'left', -fill => 'both', -expand => 'yes');
$lst_scrl->pack(-side => 'right', -fill => 'y', -expand => 'yes');
$middle->pack(-fill => 'both', -expand => 'yes');


# Bottom frame of ptk_mailer

$bottom = $main->Frame;
$msg_text = $bottom->Text(-width => 80, -height => 40,
			  -relief => 'sunken', -borderwidth => 3);
$bdy_scrl = $bottom->Scrollbar(-command => ['yview', $msg_text]);
$msg_text->configure(-yscrollcommand => ['set', $bdy_scrl]);
$msg_text->pack(-side => 'left', -fill => 'both', -expand => 'yes');
$bdy_scrl->pack(-side => 'right', -fill => 'y', -expand => 'yes');
$bottom->pack(-fill => 'both', -expand => 'yes');


# THE MAIN EVENT
MainLoop;


### Helper Functions

sub popup_auth_window {

    $tl = $main->Toplevel;
    $lw = $tl->Frame;
    $lw->Label(-text => 'Username: ')->pack(-side => 'left');
    $un = $lw->Entry(-relief => 'sunken', -borderwidth => 2);
    $un->pack(-side => 'left');
    $lw->Label(-text => 'Password: ')->pack(-side => 'left');
    $pa = $lw->Entry(-relief => 'sunken', -borderwidth => 2,
		     -show => 'x');
    $pa->pack(-side => 'left');
    $lw->pack;
    
    $bt = $tl->Frame;
    $bt->Button(-text => 'Login', -relief => 'raised', -borderwidth => 2,
		-command => sub{&user_auth()})->pack(-side=>'left');
    $bt->Button(-text => 'Cancel', -relief => 'raised', -borderwidth => 2, 
		-command => sub{$tl->withdraw})->pack(-side=>'left');
    $bt->Button(-text => 'Exit', -relief => 'raised', -borderwidth => 2,
		-command => sub{exit})->pack(-side => 'left');
    $bt->pack;
    
}

sub user_auth {
    $pop = new Mail::POP3Client($un->get, $pa->get, $host);
    if(pop_login() == 1) {
	$tl->withdraw;
	$li->pack('forget');
	$li = $top->Button(-text => 'Logout', 
			   -relief => 'raised', 
			   -borderwidth => 2, 
			   -command => sub{&logout_user});
	$li->pack(-side => 'left');
	for ($i = 1; $i <= $pop->Count; $i++) {
	    foreach ($pop->Head($i)) {
		/^(From): / and $frm = $_;
		/^(Subject): / and $subj = $_;
	    }
	    $frm = $i.". ".$frm." ".$subj;
	    $msg_area->insert('end', $frm);
	}

    } else {
	$tl->withdraw;
	$err_text = "Login Failed, or host not running POP.  Try again?";
	$err_dlg = $main->Dialog(
				 -title => 'Login Failed.',
				 -text  => $err_text,
				 -default_button => 'Exit',
				 -buttons        => ['Exit','Yes']
				 );
	$choice = $err_dlg->Show;
	if($choice eq 'Yes') {
	    &popup_auth_window();
	} else {
	    exit;
	}
    }
}


sub pop_login {
    if($pop->Count != -1) {
	return 1;
    } else {
	return 0;
    }
}


sub show_message_body {
    my @message = $msg_area->curselection;
    for(@message) { 
	$get_msg = $msg_area->get($_);
	$get_msg =~ /\d\.\sFrom:/; $get_msg = $&; $get_msg =~ s/\D//g;
	@lines = $pop->Retrieve($get_msg);
	$msg_text->delete('0.0', 'end');
	$txt = join("\n", @lines); $txt =~ s/\r//g;
	$msg_text->insert('0.0', $txt);
    }
}


sub save_message_to_file {
    $savewin = $main->Toplevel;
    
    $save_top = $savewin->Frame;
    $save_txt = $save_top->Label(-text => 'Filename:');
    $save_txt->pack(-side => 'left');
    $save_nm = $save_top->Entry;
    $save_nm->pack;
    $save_top->pack;
    
    $save_btm = $savewin->Frame;
    $save_cncl = $savewin->Button(-text => 'Cancel Save',
				  -command => sub{$savewin->withdraw});
    $save_cncl->pack(-side => 'left');
    $save_ok = $savewin->Button(-text => 'Save As', -command => sub{
	if(-e $save_nm->get) {
	    # file exists, confirm overwrite
	} else {
	    open(OUT, ">>".$save_nm->get)
		or die("Cannot open file for writing: $!");
	    print OUT $msg_text->get('0.0', 'end');
	    close(OUT);
	    $savewin->withdraw;
	}
    });
    $save_ok->pack(-side => 'left');
    $save_btm->pack;
}


sub delete_message {
    my @message = $msg_area->curselection;
    for(@message) { 
	$get_msg = $msg_area->get($_);
	$get_msg =~ /\d\.\sFrom:/; $get_msg = $&; $get_msg =~ s/\D//g;
	$pop->Delete($get_msg);
	$msg_area->delete('active');
	$msg_area->update;
	$msg_text->delete('0.0', 'end');
    }
}


sub logout_user {
    $pop->Close;
    $li->pack('forget');
    $li = $top->Button(-text => 'Login', -relief => 'raised', 
		       -command => sub{&popup_auth_window()}, 
		       -borderwidth => 2)->pack();
    $li->pack(-side=>'left');
    $msg_area->delete(0, 'end');
    $msg_text->delete('0.0', 'end');
}


### Program Usage
sub usage {
    print("Usage: $0 -host <pop host>\n");
    exit(0);
}

