#!/usr/local/bin/perl -w

use File::Basename;
use Getopt::Long;
use Mail::Cap;
use strict;

# remember, we're using strict
my($infile, $capfile, $file_ext, $mimetype, $mailcap);

# program must be executed like: programname -file inputfile
# if not, output a usage message
GetOptions('file=s', \$infile);
$infile || &usage();

# specific capfile.  without declaring this, and creating an
# object like: $mailcap = new Mail::Cap, Mail::Cap will expect
# that you have an existing mailcap file (in /etc/mailcap or such),
# or have defined an environment variable MAILCAPS
$capfile = 'mailcap';

# From File::Basename, grab the extension of the file (field 2).
# This extension will be passed to lookup_type-by_extension, which 
# returns a content-type if the extension matches an entry in the
# mime.types file
($file_ext) = (fileparse($infile, '\..*'))[2];

$mailcap = new Mail::Cap $capfile;
$mimetype = lookup_type_by_extension($file_ext);

$mailcap->view($mimetype, $infile);

sub lookup_type_by_extension {

    my($ext) = @_;
    my $m_types = 'mime.types';
    my($ctype, @ftype, @found);

    open(TFILE, $m_types) || die("error: $!");
    while(<TFILE>) {

	# remove newlines
	chop;

	# get the content-type and extensions (file types) of
	# mime.types entries by splitting each line of the file
	# on the tab character
	($ctype, @ftype) = split(/\t/, $_, 2);

	# check to see if extension appears in @ftype array
	@found = grep(/$ext/, @ftype);

	# if extension exists, there will be an element in @found,
	# so $#found+1 will be greater than zero.  return the
	# content-type of the mime.types entry
	return $ctype if $#found+1 > 0;
    }
    close(TFILE);

}

sub usage {
    print("Usage: $0 -file <filename>\n");
    exit(0); # failure
}
