#!/usr/local/bin/perl -w

use vars qw($opt_h, $opt_t, $opt_s, $opt_m);
use Getopt::Std;
use Net::Ping;

getopt('htsm');

$| = 1;

if($opt_h && $opt_t) { # -h and -t options are required.

    $opt_m = 1 unless $opt_m; # Init empty values to 1
    $opt_t = 1 unless $opt_t; # Init empty values to 1
    $opt_s = 1 unless $opt_s; # Init empty values to 1

    &usage() if $opt_t !~ /\d/; # timeout must be an integer, otherwise fail.
    &usage() if $opt_s !~ /\d/; # interval must be an integer, otherwise fail.

    for(1 .. $opt_m) {
	&ping_host($opt_h, $opt_t, $opt_s);
    }

} else {
    &usage();
}

sub ping_host {

    my($host, $timeout, $rest) = @_; # host, timeout, sleep time

    $rest = 1 unless $rest; # default interval is one second

    if(pingecho($host, $timeout)) {
	print $host, " responded.\n";
    } else {
	print $host, " is unreachable.  Try again later.\n";
    }

    sleep($rest); # sleep $rest seconds (1 by default)

}

sub usage {
    
    # Correct the command-line impaired.  :-)

    print("Usage: $0 -h <hostname> -t <timeout> -s <interval> -m <max trips>",
	  "\n\t\t\t\t",
	  "-t timeout *must* be an integer.\n\t\t\t\t",
	  "-s interval *must* be an integer.\n");
    exit(0);

}
