#!/usr/local/bin/perl -w

use Getopt::Long;
use vars qw($result $opt_u $opt_o $opt_stream);
use strict;

my($encoder, $decoder); # remember, we're using strict
$encoder = '/usr/bin/uuencode'; # installed uuencoder 
                                # later examples will use Convert::UU
$decoder = '/usr/bin/uudecode'; # installed uudecoder 
                                # later examples will use Convert::UU

$result = GetOptions qw(-u=s -o=s --stream=s);

if($opt_u && $opt_o) { # both options have been entered - proceed.
    open(OUT, ">".$opt_o) || die("Cannot open outfile: $!");
    $opt_stream = 'encode' unless $opt_stream; # assign $opt_stream if empty
    if($opt_stream eq 'encode') {
	open(ENC, $encoder." $opt_u $opt_u |") || die("Encoder error: $!");
	while(<ENC>) {
	    print OUT $_; # output the encoded lines to the outfile
	}
	close(ENC);
    } elsif($opt_stream eq 'decode') {
	open(IN, $opt_u) || die("infile error: $!");
	open(DEC, "| ".$decoder) || die("Encoder error: $!");
	while(<IN>) {
	    $_ =~ s/begin (\d+) (.+)/begin $1 $opt_o/g if $_ =~ /begin/;
	    print DEC $_;
	}
	close(DEC);
	close(IN);
    } else {
	&usage();
    }
    close(OUT);
} else { # something's wrong with the command-line options - error.
    &usage();
}

sub usage {
    print("Usage: $0 -u <infile> -o <outfile>\n",
	  "                               --stream=encode (default)\n",
	  "                               --stream=decode\n");
    exit(0); # failure
}
