#!/usr/local/bin/perl -w

use Tk;

my $m = MainWindow->new(); # Main Window

# Top Frame (contains text area w/scrollbar)
my $top = $m->Frame();
my $l = $top->Text(-relief => 'sunken', borderwidth => 2,
		 -setgrid => 'yes');
my $sb = $top->Scrollbar(-command => ['yview', $l], -relief => 'raised', 
		       -borderwidth => '2');
$l->configure(-yscrollcommand => ['set', $sb]);
$l->pack(-side => 'left', -fill => 'both', -expand => 'yes');
$sb->pack(-side => 'right', -fill => 'y', -expand => 'yes');
$top->pack(-fill => 'both', -expand => 'yes');


# Bottom frame (contains entry area which gets updated by events)
$msg = $m->Frame();
$ev_msg = $msg->Label(-text => '<Text area is empty>');
$ev_msg->pack(-fill => 'both', -expand => 'yes');


# Middle frame (contains buttons for open, close, and quit [program])
my $buttons = $m->Frame();
$b1 = $buttons->Button(-text => 'Open', -command => sub{ 
    # Creates a Toplevel window prompting user for a filename.
    # Open button: attempts to open a file.  If failed, message
    # will be updated in the bottom frame.  Window will be withdrawn
    # upon entry.  Message will be updated on success
    # Close button: Closes the Toplevel window (uses withdraw).
    # Quit button: exits the entire application

    $infile = $m->Toplevel(-title => 'Enter a Filename'); # create Toplevel

    # Create Label and text entry in top Frame of Toplevel window.
    my $base = $infile->Frame();
    $top = $base->Frame();
    $lbl = $top->Label(-text => 'Filename: ');
    $lbl->pack(-side => 'left', -fill => 'both', -expand => 'yes');
    $txt = $top->Entry();
    $txt->pack(-side => 'left', -fill => 'both', -expand => 'yes');
    $top->pack(-fill => 'both', -expand => 'yes');

    # Create Button(s) in bottom Frame of Toplevel window
    $bottom = $base->Frame();
    $yesbt = $bottom->Button(-text => 'Open File', -command => sub{
	$infile->withdraw; # Remove Toplevel on selection of Open
	$l->delete('0.0', 'end'); # Clear text area before re-writing
	if($txt->get ne '') { # Only open file if Toplevel entry was filled.
	    open(F, $txt->get);
	    while(<F>) { $l->insert('end', $_); } # Insert into text area.
	    close(F);
	}
	&status_message($ev_msg, $txt->get); # Now, update the status message
    });
    $yesbt->pack(-side => 'left');
    # Remove Toplevel window if Cancel was selected
    $nobt = $bottom->Button(-text => 'Cancel', 
			    -command => [$infile => 'withdraw']);
    $nobt->pack(-side => 'left', -fill => 'both', -expand => 'yes');
    $quit = $bottom->Button(-text => 'Quit', -command => sub{exit});
    $quit->pack(-side => 'left', -fill => 'both', -expand => 'yes');
    $bottom->pack(-fill => 'both', -expand => 'yes');
    $base->pack(-fill => 'both', -expand => 'yes');
});
$b1->pack(-side => 'left', -fill => 'both', -expand => 'yes');
$b2 = $buttons->Button(-text => 'Clear', -command => sub {
    &status_message($ev_msg, '');
    $l->delete('0.0', 'end');
});
$b2->pack(-side => 'left', -fill => 'both', -expand => 'yes');
$b3 = $buttons->Button(-text => 'Quit', -command => sub{exit});
$b3->pack(-side => 'left', -fill => 'both', -expand => 'yes');
$buttons->pack(-fill => 'both', -expand => 'yes');

$msg->pack(-side => 'left', -fill => 'both', -expand => 'yes');

# THE MAIN EVENT
MainLoop;

# Helper functions
sub status_message {
    
    my($message, $fn) = @_;
    my($all);
    
    $all = undef;
    if($fn ne '') {
	$all = "Opening: ".$fn;
    } else {
	$all = '<Text area is empty>';
    }

    $ev_msg->pack('forget');
    $ev_msg = $msg->Label(-text => $all);
    $ev_msg->pack();
    $msg->update;

}

